<?php

namespace Drupal\link_to_entity\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'Link_Entity_Belong_to' formatter.
 *
 * @FieldFormatter(
 *   id = "Link_Entity_Belong_to",
 *   label = @Translation("Link entity belongs to"),
 *   field_types = {
 *     "entity_reference",
 *   },
 * )
 */
class LinkEntityBelongTo extends EntityReferenceFormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = array();
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $item) {
      $label = $item->label();
      $entity = $items->getEntity();
      $entity_type = $entity->getEntityType()->id();
      $uri = Url::fromRoute('entity.' . $entity_type . '.canonical', [$entity_type => $entity->id()]);
      $elements[$delta] = [
        '#type' => 'link',
        '#title' => $label,
        '#url' => $uri,
        '#options' => $uri->getOptions(),
      ];

      $elements[$delta]['#cache']['tags'] = $item->getCacheTags();
    }

    return $elements;
  }

}
